//Source file for Attribute functions.

#include <attr.h>
#include <log.h>
#include <util.h>

//Unload AttributeMap 
void unload_attrmap(attrmap *givam){
	if(givam){
		if(givam->traindata){
			free(givam->traindata);
			givam->traindata=NULL;
		}
		givam->ntrains=0;
		givam->width=0;	givam->height=0;
		givam->timesec=0;
		givam->tilewidth=0; givam->tileheight=0;
		free(givam);
		#ifdef VERBOSE
			log_printf("Freed AttributeMap");
		#endif //VERBOSE
	}
}

//Create AttributeMap 
attrmap *create_attrmap(int width, int height, int ntrains){
	attrmap *newam=NULL;	
	int allocsizetdata=0;
	if(width<=0||height<=0||ntrains<=0) return newam;
	newam=malloc(sizeof(attrmap));	
	if(newam) {
		newam->width=width; newam->height=height;
		newam->tilewidth=0; newam->tileheight=0; //Remember to read it sometime ;)
		newam->timesec=0; 
		newam->ntrains=ntrains;		
		allocsizetdata=sizeof(int)*ntrains;
		newam->traindata=malloc(allocsizetdata);
		if(newam->traindata) {
			memset(newam->traindata,ATTR_NULL,allocsizetdata); //Initialize values
			#ifdef VERBOSE
				log_printf("Created AttributeMap");
			#endif //VERBOSE	
		}
		else  {
			unload_attrmap(newam); newam=NULL;
		}
	}	
	return newam;
}

//Load AttributeMap from a pre opened file
attrmap *load_attrmap_file(FILE *fattr){	
	int i,j,l;
	int width=0,height=0,ntrains=0;	
	int tilewidth=0,tileheight=0;
	attrmap *newam=NULL;
	
	fscanf(fattr,"%d%d%d%d%d",&width,&height,&tilewidth,&tileheight,&ntrains);

	newam=create_attrmap(width,height,ntrains);

	if(newam){
		newam->tilewidth=tilewidth;
		newam->tileheight=tileheight;
		for(i=0;i<newam->ntrains;++i){
			fscanf(fattr,"%d",newam->traindata+i);
		}
		fscanf(fattr,"%d%d%d%d",&newam->r,&newam->g,&newam->b,&newam->a);
		fscanf(fattr,"%d",&newam->timesec);
		#ifdef VERBOSE
			log_printf("Loaded AttributeMap");
		#endif //VERBOSE
	}

	return newam;	
}