//Source file for driver program.

#include <header.h>
#include <global.h>
#include <game.h>

//Speed Counter for Timer.
volatile long speed_counter=0;

//Flag for Initialized.
int inited=0;

//Flag for Loaded.
int loaded=0;

//Flag for Quiting Loop.
int quitloop=0;

//Termination Function for Game
void terminate_loop(){
	++quitloop;
}

//Function called by timer.
void increment_speed_counter()
{
	speed_counter++;
}
END_OF_FUNCTION(increment_speed_counter); 

//Uninitializes System.
void quit(){
	if(inited) { 
		//Quit Allegro
		allegro_exit(); 
		log_printf("Allegro UnInitialized");
	}
	if(soundsystem) --soundsystem;
	//Uninitialize Logging
	quit_log();	
	if(inited) --inited; 
}

//Initializes System.
int init(){
	int status=-1;
	quit();

	//Initialize Logging
	init_log();

	//Initialize Allegro
	if((status=allegro_init())) return status;
	log_printf("Allegro Initialized");

	//Initialize Window Title
	set_window_title(APPTITLE);

	//Initialize Timer
	if((status=install_timer())) return status;
	log_printf("Timer Initialized");

	//Initialize Keyboard
	if((status=install_keyboard())) return status;
	log_printf("Keyboard Initialized");

	//Initialize Screen
	set_color_depth(bpp);
	if(windowed_mode) {
		if((status=set_gfx_mode(GFX_AUTODETECT_WINDOWED,xres,yres,0,0))) return status;
		log_printf("Screen Initialized: %d x %d @ %d bpp Windowed",xres,yres,bpp);
	} else {
		if((status=set_gfx_mode(GFX_AUTODETECT_FULLSCREEN,xres,yres,0,0))) return status;
		log_printf("Screen Initialized: %d x %d @ %d bpp FS",xres,yres,bpp);
	}

	//Initialize Sound System
	if(status=install_sound(DIGI_AUTODETECT,MIDI_AUTODETECT,NULL)) {
		soundsystem=0;
		log_printf("Sound system disabled");
		install_sound(DIGI_NONE, MIDI_NONE, NULL);
	}
	else {
		log_printf("Sound system Initialized");
		++soundsystem;	
	}     
	
	//Initialize Random Number Generator
	srand(time(NULL));
	log_printf("Initialized Random Number Seed");

	clear_bitmap(screen); 
 	++inited;
	return 0;
}

//Unloads Data.
void unload(){
	if(dbuffer) {
		//Destroy DBuffer
		destroy_bitmap(dbuffer); dbuffer=NULL;
		log_printf("Destroyed DBuffer");
	}
	if(loaded) --loaded;
}

//Loads Data.
int load(){
	int status=-1;
	unload();
	
	//Create DBuffer
	dbuffer=create_bitmap(xres,yres);
	if(!dbuffer) return status;
	log_printf("Created DBuffer");


	++loaded;	
	return 0;
}

//Main Loop Quit
void loop_quit(){
	//Quit Game
	log_printf("UnInitializing Game");
	game_quit();
	log_printf("UnInitialized Game");

	log_printf("Quitting Loop");
}

//Main Loop Init
int loop_init(){
	int status=-1;
	LOCK_VARIABLE(speed_counter);
	LOCK_FUNCTION(increment_speed_counter);
	//Set Timer
        install_int_ex(increment_speed_counter, BPS_TO_TIMER(lps));

	log_printf("Creating Loop at %d LPS",lps);
	quitloop=0;
	
	//Initialize Game
	log_printf("Initializing Game");
		if(status=game_init()) return status;	
	log_printf("Initialized Game");
	return 0;
}

//Game Loop Init
void loop_run(){
	while(!quitloop){	
		game_input();
   		while(speed_counter > 0){
			game_logic();
			speed_counter--;
		}
		game_render();
 	}
}

//Main
int main(){
	if(init()) ;
	else {
		if(load()) ;
		else {			
			if(loop_init()) ;
			else 
				loop_run();
			loop_quit();
		}
		unload();
	}
	quit();
	
	return 0;
}
END_OF_MAIN();